% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of Takagi-
% Sugeno-Kang (TSK) type for the data in Echavarria-Heras et al 2018.

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Numerical precision

  format longe
  
% Reading the data of the Echavarria-Heras et al. 2015

  load Echavarria_Heras_et_al_2018_Raw_data.txt
    
  A = [Echavarria_Heras_et_al_2018_Raw_data(:,1), Echavarria_Heras_et_al_2018_Raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
    
% ********************************************
% Building the TSK fuzzy inference model
% ********************************************
      
  radii = 0.5416;  % Specifies a cluster center's range of influence

  fismat = gentskfistrain(X,Y,radii);
  
  if (isempty(fismat) == 1)
      return;
  end
      
  printprms('Mean Response Mean', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************

 [Yh,g,phi,alpha] = evaltskfis(X,fismat,'comp2gaussmf'); 
  
% ********************************************
% Mean Response Curve
% ********************************************  
      
  load xyhet.txt
  
  figure; % Figure 12b
  plot(X,Y,'oblack', X,Yh,'-b', xyhet(:,1), xyhet(:,2), '-r');
  ylabel('y');
  xlabel('x') 
  legend('Observed','E_{aTSK}(y|x)','E_{gTSK}(y|x)');
  title('Figure 12b');
  
% ********************************************  
% Reproducibility indeices
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(Y, Yh, 8);
  
  printdata('Table 8. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
   
  ei = Y - Yh;
  
  figure; % Figure 12c.
  hold on
  plot(X,ei,'ob');
  line([0 8000],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 12c');
  hold off

% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 12d.
  h = qqplot(Yh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 12d')
    
% ********************************************  
% Evaluate TSK MODEL FIS in continuos data
% ********************************************

  cX  = min(X):((max(X)-min(X))/10000):max(X);
  cX  = cX';

  [yp,g,phi,alpha] = evaltskfis(cX,fismat,'comp2gaussmf'); 

% ********************************************
% Antecedents
% ********************************************  

  figure; % Figure
  hold on
  plotmf(fismat,'input',1,10000);
  axis([0 500 0 1]); 
  line([231.71 231.71], [0 1], 'Color', [0.2, 0.2, 0.2])
  line([231.71 475.0944815], [0.65 0.65], 'Color', [0.2, 0.2, 0.2])
  ylabel('Grade of Membership');
  xlabel('x') 
  legend('\mu_1 (x)','\mu_2 (x)');
  hold off
  
% ********************************************
% Normalized firing strength
% ********************************************  

  figure; % Figure 12a
  hold on
  plot(cX,phi(:,1),'-r', cX,phi(:,2),'-b');
  axis([0 500 0 1])
  ylabel('Normalized firing strength');
  xlabel('x') 
  legend('\vartheta^{ 1}(x)','\vartheta^{ 2}(x)');
  title('Figure 12a');
  hold off
  
% ********************************************
% Consequent functions
% ******************************************** 

  figure; % Figure
  plot(cX,g(:,1),'-r',cX,g(:,2),'-b');
  axis([0 500 0 +inf])
  ylabel('Consequent functions');
  xlabel('x') 
  legend('f^{ 1}(x) = ln\beta_1 + \alpha_1 x','f^{ 2}(x) = ln\beta_2 + \alpha_2 x');
    
% ********************************************
% Component products
% ******************************************** 

  figure; % Figure
  plot(cX,phi(:,1).*g(:,1),'-r',cX,phi(:,2).*g(:,2),'-b');
  axis([0 300 0 +inf])
  ylabel('Component products');
  xlabel('x') 
  legend('\vartheta^{ 1}(x) f^{ 1}(x)','\vartheta^{ 2}(x) f^{ 2}(x)');
