% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of Takagi-
% Sugeno-Kang (TSK) type for the data in Huxley 1932

% Cleaning variables resident in the memory and screen

  clc
  clearvars;
  
% Numerical precision

  format longe

% Reading the data of the Echavarria-Heras et al. 2015

  load Huxley_1932_Raw_data.txt
    
  A = [Huxley_1932_Raw_data(:,1), Huxley_1932_Raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
    
% ********************************************
% Building the TSK fuzzy inference model
% ********************************************

  radii = 0.668;  % Specifies a cluster center's range of influence
  
  fismat = gentskfistrain(X,Y,radii);
  
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************

  [Yh,g,phi,alpha] = evaltskfis(X,fismat,'comp2gaussmf');
    
% ********************************************
% Mean Response Curve
% ********************************************  
   
  load bYhpla.txt 
  
  figure; % Figure 14b
  hold on
  plot(X,Y,'oblack', X,Yh,'-b',bYhpla(:,1),bYhpla(:,2),'-r');
  line([340.801177 340.801177],[0 200], 'Color', [0.2, 0.2, 0.2]);
  ylabel('y');
  xlabel('x') 
  legend('Observed','E_{aTSK}(y|x)','E_{gTSK}(y|x)');
  title('Figure 14b');
  hold off
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(Y, Yh, 8);
  
  printdata('Table 10. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************
% Residuals
% ********************************************
   
  ei = Y - Yh;
  
  figure; % Figure 14c.
  hold on
  plot(X,ei,'ob');
  line([0 2500],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 14c');
  hold off

% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 14d.
  h = qqplot(Yh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 14d')
  
% ********************************************  
% Evaluate TSK MODEL FIS in continuos data
% ********************************************

  cX  = min(X):((max(X)-min(X))/10000):max(X);
  cX  = cX';
  
  [Yh,g,phi,alpha] = evaltskfis(cX,fismat,'comp2gaussmf');
  
% ********************************************
% Antecedents
% ********************************************  
    
  figure; % Figure
  hold on
  plotmf(fismat,'input',1,10000);
  axis([0 2500 0 1]); 
  
  line([164.52 164.52], [0 1], 'Color', [0.2, 0.2, 0.2])
  line([705.8768 705.8768], [0 1], 'Color', [0.2, 0.2, 0.2])
  
  line([164.52 292], [0.6 0.6], 'Color', [0.2, 0.2, 0.2])
  line([705.8768 1255], [0.6 0.6], 'Color', [0.2, 0.2, 0.2])
  
  ylabel('Grade of Membership');
  xlabel('x') 
  legend('\mu_{\phi_1}(x)','\mu_{\phi_2}(x)');
  hold off
  
% ********************************************
% Normalized firing strength
% ********************************************  

  figure; % Figure 14a
  hold on
  plot(cX,phi(:,1),'-r', cX,phi(:,2),'-b');
  axis([0 2500 0 1])
  ylabel('Normalized firing strength');
  xlabel('x') 
  legend('\vartheta^{ 1}(x)','\vartheta^{ 2}(x)');
  title('Figure 14a');
  hold off
  
% ********************************************
% Consequent functions
% ******************************************** 

  figure; % Figure
  plot(cX,g(:,1),'-r',cX,g(:,2),'-b');
  axis([0 2500 0 +inf])
  ylabel('Consequent functions');
  xlabel('x') 
  legend('f^{ 1}(x) = ln\beta_1 + \alpha_1 x','f^{ 2}(x) = ln\beta_2 + \alpha_2 x');
    
% ********************************************
% Component products
% ******************************************** 

  figure; % Figure
  plot(cX,phi(:,1).*g(:,1),'-r',cX,phi(:,2).*g(:,2),'-b');
  ylabel('Component products');
  xlabel('x') 
  legend('\vartheta^{ 1}(x) f^{ 1}(x)','\vartheta^{ 2}(x) f^{ 2}(x)');

