% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of Takagi-
% Sugeno-Kang (TSK) type for the data in Mascaro et al 2011

% Cleaning variables resident in the memory and screen

  clc
  clearvars;
  
% Numerical precision

  format longe

% Reading the data of the Mascaro et al. 2011

  load Mascaro_et_al_2011_Raw_data.txt
    
  A = [Mascaro_et_al_2011_Raw_data(:,1), Mascaro_et_al_2011_Raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
  
% ********************************************  
% Building the TSK fuzzy inference model
% ********************************************

  radii = 0.8553;  % Specifies a cluster center's range of influence
  
  fismat = gentskfistrain(X,Y,radii);
  
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
   [Yh,g,phi,alpha] = evaltskfis(X,fismat,'comp2gaussmf');
  
% ********************************************
% Mean Response Curve
% ********************************************  
       
  load bYhpla.txt;
  
  figure; % Figure 13b
  plot(X,Y,'oblack', X,Yh,'-b', bYhpla(:,1),bYhpla(:,2),'-r');
  ylabel('y');
  xlabel('x') 
  legend('Observed','E_{\ita\rmTSK}(\ity \rm| \itx\rm)','E_{\itg\rmTSK}(\ity \rm| \itx\rm)');
  title('Figure 13b');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(Y, Yh, 8);
  
  printdata('Table 9. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
   
  ei = Y - Yh;
  
  figure; % Figure 13c.
  hold on
  plot(X,ei,'ob');
  line([0 35],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 13c');
  hold off

% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 13d.
  h = qqplot(Yh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 13d')
  
% ********************************************  
% Evaluate TSK MODEL FIS in continuos data
% ********************************************

  cX  = min(X):((max(X)-min(X))/10000):max(X);
  cX  = cX';
 
  [Yh,g,phi,alpha] = evaltskfis(cX,fismat,'comp2gaussmf');
  
% ********************************************
% Antecedents
% ********************************************  
    
  figure;
  hold on
  plotmf(fismat,'input',1,10000);
  axis([min(X) +inf 0 +inf]) 
  ylabel('Grade of Membership');
  xlabel('x') 
  legend('\mu_1 (x)','\mu_2 (x)');
  hold off
   
% ********************************************
% Normalized firing strength
% ********************************************  

  figure; % Figure 13a
  hold on
  plot(cX,phi(:,1),'-r', cX,phi(:,2),'-b');
  axis([min(X) +inf 0 +inf])
  ylabel('Normalized firing strength');
  xlabel('x') 
  legend('\vartheta^{ 1}(x)','\vartheta^{ 2}(x)');
  title('Figure 13a');
  hold off
    
% ********************************************
% Consequent functions
% ******************************************** 

  figure; % Figure
  plot(cX,g(:,1),'-r',cX,g(:,2),'-b');
  axis([min(X) +inf min(Y) +inf])
  ylabel('Consequent functions');
  xlabel('x') 
  legend('f^{ 1}(x) = ln\beta_1 + \alpha_1 x','f^{ 2}(x) = ln\beta_2 + \alpha_2 x');
    
% ********************************************
% Component products
% ******************************************** 

  figure; % Figure
  plot(cX,phi(:,1).*g(:,1),'-r',cX,phi(:,2).*g(:,2),'-b');
  ylabel('Component products');
  xlabel('x') 
  legend('\vartheta^{ 1}(x) f^{ 1}(x)','\vartheta^{ 2}(x) f^{ 2}(x)');

