% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of Takagi-
% Sugeno-Kang (TSK) type for the data in Robertis and Williams 2018

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data set

  load DeRobertis_and_Williams_Raw_data.txt
    
  A = [DeRobertis_and_Williams_Raw_data(:,1), DeRobertis_and_Williams_Raw_data(:,2)];
    
% Data

  X = exp(A(:,1));
  Y = exp(A(:,2));
      
% ********************************************
% Building the TSK fuzzy inference model
% ********************************************  
  
  radii = 0.22;  % Specifies a cluster center's range of influence
  
  fismat = gentskfistrain(X,Y,radii);
  
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************
% Mean Response Curve
% ********************************************  
  
  [i,j] = sort(X);
  cX = i;
  cY = Y(j);
  
  [cYh,g,phi,alpha] = evaltskfis(cX,fismat,'comp2gaussmf'); 
 
  load brobpla.txt
  
  figure; % Figure 15a
  plot(X,Y,'oblack', cX,cYh,'ob', brobpla(:,1), brobpla(:,2),'or');
  ylabel('y');
  xlabel('x') 
  legend('Observed','E_{aTSK}(y|x)','E_{gTSK}(y|x)');
  title('Figure 15a');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [Yh,g,phi,alpha] = evaltskfis(X,fismat,'comp2gaussmf'); 
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(Y, Yh, 8);
  
  printdata('Table 11. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
     
  ei = Y - Yh;
  
  figure; % Figure 15b.
  hold on
  plot(Yh,ei,'om');
  line([0 3000],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 15b');
  hold off

% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 15c.
  h = qqplot(Yh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 15c')
  
% ********************************************  
% Evaluate TSK MODEL FIS in continuos data
% ********************************************

  cX  = min(X):((max(X)-min(X))/10000):max(X);
  cX  = cX';

  [yp,g,phi,alpha] = evaltskfis(cX,fismat,'comp2gaussmf'); 

% ********************************************
% Antecedents
% ********************************************  
  
  figure;
  plotmf(fismat,'input',1,10000);
  ylabel('Grade of Membership');
  xlabel('x') 
  legend('\mu_1 (x)','\mu_2 (x)','\mu_3 (x)');
  
% ********************************************
% Normalized firing strength
% ********************************************  

  figure; % Figure
  plot(cX,phi(:,1),'-r', cX,phi(:,2),'-b', cX,phi(:,3),'-black');
  ylabel('Normalized firing strength');
  xlabel('x') 
  legend('\vartheta^{ 1}(x)','\vartheta^{ 2}(x)','\vartheta^{ 3}(x)');

  
% ********************************************
% Consequent functions
% ******************************************** 

  figure; % Figure
  plot(cX,g(:,1),'-r', cX,g(:,2),'-b', cX,g(:,3),'-black');
  ylabel('Consequent functions');
  xlabel('x') 
  legend('f^{ 1}(x) = ln\beta_1 + \alpha_1 x','f^{ 2}(x) = ln\beta_2 + \alpha_2 x',...
          'f^{ 3}(x) = ln\beta_3 + \alpha_3 x');
    
% ********************************************
% Component products
% ******************************************** 

  figure; % Figure
  plot(cX,phi(:,1).*g(:,1),'-r',cX,phi(:,2).*g(:,2),'-b',cX,phi(:,3).*g(:,3),'-b');
  ylabel('Component products');
  xlabel('x') 
  legend('\vartheta^{ 1}(x) f^{ 1}(x)','\vartheta^{ 2}(x) f^{ 2}(x)',...
         '\vartheta^{ 3}(x) f^{ 3}(x)');
