% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of 
% Takagi-Sugeno-Kang (TSK) type for the data in Echavarria-Heras et al 2018. 

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data set

  load Echavarria_Heras_et_al_2018_raw_data.txt
    
  A = [Echavarria_Heras_et_al_2018_raw_data(:,1), Echavarria_Heras_et_al_2018_raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
  
% Logarithmic transformation of the zostera data set

  lnX = log(X);
  lnY = log(Y);
  
% ********************************************
% Observed data
% ********************************************
       
  figure; % Figure 1a.
  plot(lnX,lnY,'or');
  ylabel('log(Leaf biomass)');
  xlabel('log(Leaf area)'); 
  title('Figure 1a');
  
% ********************************************  
% Building the TSK - PLA fuzzy inference model
% ********************************************
  
  radii = 0.47;  % specifies a cluster center's range of influence
  
  fismat = gentskfis(lnX, lnY, radii);
 
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
  [lnYh,g,phi,alpha] = evaltskfis(lnX,fismat,'gaussmf');
   
% ********************************************
% Mean response curve
% ********************************************
       
  lnYh = evalfis(lnX,fismat);
            
  figure; % Figure 4a.
  plot(lnX,lnY,'or', lnX, lnYh,'ob');
  legend('Observed','v_{TSK}(u,\pi)');
  ylabel('log(Leaf biomass)');
  xlabel('u'); 
  title('Figure 4a');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(lnY, lnYh, 8);
  
  printdata('Table 4. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
  
  ei = lnY - lnYh;
   
  figure; % Figure 4b.
  hold on
  plot(lnYh,ei, 'bo');
  line([min(lnYh) max(lnYh)],[0,0])
  xlabel('u');
  ylabel('Residuals');
  title('Figure 4b')
  hold off
  
% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 4c.
  h = qqplot(lnYh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 4c')
      
% ******************************************** 
% Back - Transformation - Mean response curve  
% ********************************************
   
  fc =  correction_factor_eq_56(lnY, lnYh, ei);
   
  figure; % Figure
  plot(X,Y,'oblack', X, exp(lnYh)*fc,'om');
  legend('Observed','E_{gTSK}(y|x)');
  xlabel('x'); 
  ylabel('y');
   
% ******************************************** 
% Back - Transformation - Mean response curve    
% Reproducibility indices
% ********************************************

  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(exp(lnY), exp(lnYh)*fc, 8);
  
  printdata('Table 8. Back - Transformation - Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************
% Back - Transformation - Mean response curve 
% Residuals
% ********************************************
   
  ei =  exp(lnY) - exp(lnYh);
  
  figure; % Figure
  hold on
  plot(exp(lnYh),ei,'ob');
  line([min(exp(lnYh)) max(exp(lnYh))],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  hold off

% ********************************************
% Back - Transformation - Mean response curve
% Residuals qq-plot
% ********************************************

  figure; % Figure
  h = qqplot(exp(lnX),ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
         
% ********************************************  
% Evaluate TSK MODEL FIS in continuous data
% ********************************************
 
  clnX  = min(lnX):((max(lnX)-min(lnX))/10000):max(lnX);
  clnX  = clnX';  
  
  [lnYh,g,phi,alpha] = evaltskfis(clnX,fismat,'gaussmf');
  
% ********************************************
% Membership functions
% ********************************************
      
  figure; % Figure 3a
  hold on
  plotmf(fismat,'input',1,10000);
  ylabel('Grade of Membership');
  xlabel('u') 
  legend('mu_1(u)','\mu_2(u)');
  title('Figure 3a');
  hold off
  
% ********************************************
% Normalized firing strength
% ********************************************
      
  figure; % Figure 3b
  hold on
  plot(clnX,phi(:,1),'-r', clnX,phi(:,2),'-b');
  line([3.90465 3.90465], [0 0.5], 'Color', [0.2, 0.2, 0.2]) 
  ylabel('Normalized firing strength');
  xlabel('u') 
  legend('\vartheta^1(u)','\vartheta^2(u)');
  title('Figure 3b');
  hold off
  
% ********************************************
% Consequent function
% ********************************************
       
  figure; % Figure 3c
  plot(clnX,g(:,1),'-r', clnX,g(:,2),'-b');
  ylabel('Consequent functions');
  xlabel('u') 
  legend('f^{ 1}(u) = ln\beta_1 + \alpha_1 u','f^{ 2}(u) = ln\beta_2 + \alpha_2 u');
  title('Figure 3c');
  
% ********************************************
% Component product
% ********************************************
    
  figure; % Figure 3d
  hold on
  plot(clnX,phi(:,1).*g(:,1),'-r', clnX,phi(:,2).*g(:,2),'-b');
  line([3.92383 3.92383], [-9.1, -3.798], 'Color', [0.2,0.2,0.2]);
  ylabel('Component products');
  xlabel('u') 
  legend('g^1(u)','g^2(u)');
  title('Figure 3d');
  hold off
