% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of 
% Takagi-Sugeno-Kang (TSK) type for the data in Echavarria-Heras et al 2018. 

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data set

  load Echavarria_Heras_et_al_2018_raw_data.txt
    
  A = [Echavarria_Heras_et_al_2018_raw_data(:,1), Echavarria_Heras_et_al_2018_raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
  
% Logarithmic transformation of the zostera data set

  lnX = log(X);
  lnY = log(Y);
  
% ********************************************  
% Building the TSK - PLA fuzzy inference model
% ********************************************
        
  radii = 0.47;  % specifies a cluster center's range of influence
  
  fismat = gentskfis(lnX, lnY, radii);
 
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
  [lnYh,g,phi,alpha] = evaltskfis(lnX,fismat,'gaussmf');
   
% ********************************************
% Mean response curve
% ********************************************
       
  lnYh = evalfis(lnX,fismat);
            
  figure;
  plot(lnX,lnY,'or', lnX, lnYh,'ob');
  legend('Observed','v_{TSK}(u,\pi)');
  ylabel('log(Leaf biomass)');
  xlabel('u'); 

  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(lnY, lnYh, 8);
  
  printdata('Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
  
  ei = lnY - lnYh;
   
  figure;
  hold on
  plot(lnYh,ei, 'bo');
  line([min(lnYh) max(lnYh)],[0,0])
  xlabel('u');
  ylabel('Residuals');
  hold off
  
% ********************************************
% Residuals qq-plot
% ********************************************

  figure;
  h = qqplot(lnYh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
      
% ******************************************** 
% Back - Transformation - Mean response curve  
% ********************************************
   
  fc =  correction_factor_eq_56(lnY, lnYh, ei);
   
  figure;
  plot(X,Y,'oblack', X, exp(lnYh)*fc,'om');
  legend('Observed','E_{gTSK}(y|x)');
  xlabel('x'); 
  ylabel('y');
   
% ******************************************** 
% Back - Transformation - Mean response curve    
% Reproducibility indices
% ********************************************

  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(exp(lnY), exp(lnYh)*fc, 8);
  
  printdata('Back - Transformation - Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************
% Back - Transformation - Mean response curve 
% Residuals
% ********************************************
   
  ei = exp(lnY) - exp(lnYh);
  
  figure;
  hold on
  plot(exp(lnYh),ei,'ob');
  line([min(exp(lnYh)) max(exp(lnYh))],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  hold off

% ********************************************
% Back - Transformation - Mean response curve
% Residuals qq-plot
% ********************************************

  figure;
  h = qqplot(exp(lnX),ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
         
% ********************************************  
% Evaluate TSK MODEL FIS in continuous data
% ********************************************
 
  clnX  = min(lnX):((max(lnX)-min(lnX))/10000):max(lnX);
  clnX  = clnX';  
  
  [lnYh,g,phi,alpha] = evaltskfis(clnX,fismat,'gaussmf');
  
% ********************************************
% Membership functions
% ********************************************
      
  figure;
  hold on
  plotmf(fismat,'input',1,10000);
  ylabel('Grade of Membership');
  xlabel('u') 
  legend('mu_1(u)','\mu_2(u)');
  hold off
  
