% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of 
% Takagi-Sugeno-Kang (TSK) type for the data in Huxley 1932.

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data set

  load Huxley_1932_raw_data.txt
    
  A = [Huxley_1932_raw_data(:,1), Huxley_1932_raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
  
% Logarithmic transformation of the zostera data set

  lnX = log(X);
  lnY = log(Y);
  
% ********************************************
% Observed data
% ********************************************
       
  figure; % Figure 1c.
  plot(lnX,lnY,'or');
  xlabel('log(Chela - wieght)')
  ylabel('log(Body - weight)');
  title('Figure 1c');
  
% ********************************************  
% Building the TSK - PLA fuzzy inference model
% ********************************************
  
  radii = 0.8;
  
  fismat = gentskfis(lnX, lnY, radii);
 
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
  [lnYh,g,phi,alpha] = evaltskfis(lnX,fismat,'gaussmf');
   
% ********************************************
% Mean response curve
% ********************************************
      
  lnYh = evalfis(lnX,fismat);
      
  figure; % Figure 8e.
  plot(lnX,lnY,'or', lnX, lnYh,'-b');
  line([5.83129925, 5.83129925], [3.5, 5.5], 'Color', [0.2 0.2 0.2]);
  ylabel('log(Body - weight');
  xlabel('u'); 
  legend('Observed','v_{TSK}(u,\pi)');
  title('Figure 8e');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(lnY, lnYh, 8);
  
  printdata('Table 6. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
   
  ei = lnY - lnYh;
  
  figure; % Figure 9b.
  hold on
  plot(lnYh,ei,'ob');
  line([0 8],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 9b');
  hold off
  
% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 9c.
  h = qqplot(lnYh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 9c')
    
% ********************************************
% Mean response curve in the scale presented
% in the Huxley's book
% ********************************************

  figure; % Figure 8d.
  plot(lnX./2.3,lnY./2.3,'or', lnX./2.3, lnYh./2.3,'-b');
  line([2.9, 2.9], [2, 3], 'Color', [0.2 0.2 0.2]);
  line([5.83129925/2.3, 5.83129925/2.3], [3.5/2.3, 5.5/2.3], 'Color', [0.2 0.2 0.2]);
  ylabel('log(Body - weight');
  xlabel('u'); 
  legend('Observed','v_{TSK}(u,\pi)'); 
  title('Figure 8d');
          
% ********************************************
% Data set presented in Figure 3
% in the Huxley's book
% ********************************************

  figure; % Figure 8d.  Huxley's book
  plot(lnX./2.3,lnY./2.3,'or');
  line([2.9, 2.9], [2, 3], 'Color', [0.2 0.2 0.2]);
  line([5.83129925/2.3, 5.83129925/2.3], [3.5/2.3, 5.5/2.3], 'Color', [0.2 0.2 0.2]);
  xlabel('log(Chela - wieght)')
  ylabel('log(Body - weight)');
  title('Figure 8d');
  
% ******************************************** 
% Back - Transformation - Mean response curve  
% ********************************************

  fc =  correction_factor_eq_56(lnY, lnYh, ei);
  
  figure; % Figure 14b
  hold on
  plot(X,Y,'oblack', X, exp(lnYh)*fc,'-m');
  line([exp( 5.83129925), exp( 5.83129925)], [0, 200], 'Color', [0.2 0.2 0.2]);
  legend('Observed','E_{gTSK}(y|x)','x_b')
  ylabel('y');
  xlabel('x'); 
  title('Figure 14b');
  hold off
  
  bYhpla = [X, exp(lnYh)*fc];
  save bYhpla.txt bYhpla -ascii
   
% ******************************************** 
% Back - Transformation - Mean response curve    
% Reproducibility indices
% ********************************************

  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(exp(lnY), exp(lnYh)*fc, 8);
  
  printdata('Table 10. Back - Transformation - Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************
% Back - Transformation - Mean response curve 
% Residuals
% ********************************************
  
  ei = exp(lnYh) - Y;
  
  figure; % Figure 14e.
  hold on
  plot(exp(lnX),ei,'ob');
  line([0 2500],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 14e');
  hold off

% ********************************************
% Back - Transformation - Mean response curve
% Residuals qq-plot
% ********************************************

  figure; % Figure 14f.
  h = qqplot(exp(lnX),ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 14f')
         
% ********************************************  
% Evaluate TSK MODEL FIS in continuous data
% ********************************************
 
  clnX  = min(lnX):((max(lnX)-min(lnX))/10000):max(lnX);
  clnX  = clnX';  
  
  [lnYh,g,phi,alpha] = evaltskfis(clnX,fismat,'gaussmf');
  
% ********************************************
% Membership functions
% ********************************************
     
  figure; % Figure
  plotmf(fismat,'input',1,10000);
  
  line([4.9022885 4.9022885], [0, 1], 'Color', [0.2 0.2 0.2]);
  line([6.66823389 6.66823389], [0, 1], 'Color', [0.2 0.2 0.2]);
  
  line([4.9022885 5.65357084], [0.7, 0.7], 'Color', [0.2 0.2 0.2]);
  line([6.66823389 7.345], [0.7, 0.7], 'Color', [0.2 0.2 0.2]);
  
  ylabel('Grade of Membership');
  xlabel('u') 
  legend('M_{\phi_1}(u)','M_{\phi_2}(u)');
  title('Figure');
  
  
% ********************************************
% Normalized firing strength
% ********************************************
      
  figure; % Figure 8a
  hold on
  plot(clnX,phi(:,1),'-r', clnX,phi(:,2),'-b');
  line([5.83129925, 5.83129925], [0, 0.5], 'Color', [0.2 0.2 0.2]);
  ylabel('Normalized firing strength');
  xlabel('u') 
  legend('\vartheta^1(u)','\vartheta^2(u)');
  title('Figure 8a');
  hold off
  
% ********************************************
% Consequent function
% ********************************************
       
  figure; % Figure 8b
  plot(clnX,g(:,1),'-r',clnX,g(:,2),'-b');
  ylabel('Consequent functions');
  xlabel('u') 
  legend('f^{ 1}(u) = ln\beta_1 + \alpha_1 u','f^{ 2}(u) = ln\beta_2 + \alpha_2 u');
  title('Figure 8b');
  
% ********************************************
% Component product
% ********************************************
    
  figure; % Figure 8c
  hold on
  plot(clnX,phi(:,1).*g(:,1),'-r',clnX,phi(:,2).*g(:,2),'-b');
  ylabel('Component products');
  xlabel('u') 
  legend('\vartheta^{ 1}(u) g^1(u)', '\vartheta^{ 2}(u) g^2(u)');
  title('Figure 8c');
  hold off
