% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of 
% Takagi-Sugeno-Kang (TSK) type for the data in Mascaro et al 2011.

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data of the Mascaro et al. 2011

  load Mascaro_et_al_2011_raw_data.txt
    
  A = [Mascaro_et_al_2011_raw_data(:,1), Mascaro_et_al_2011_raw_data(:,2)];
    
% Data

  X = A(:,1);
  Y = A(:,2);
  
% Logarithmic transformation of the zostera data set

  lnX = log(X);
  lnY = log(Y);
  
% ********************************************
% Observed data
% ********************************************
       
  figure; % Figure 1b.
  plot(lnX,lnY,'or');
  ylabel('log(Aboveground Biomass)');
  xlabel('log(Diameter)'); 
  title('Figure 1b');
  
% ********************************************  
% Building the TSK - PLA fuzzy inference model
% ********************************************
   
  radii = 0.8;
  
  fismat = gentskfis(lnX, lnY, radii);
 
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
  [lnYh,g,phi,alpha] = evaltskfis(lnX,fismat,'gaussmf');
   
% ********************************************
% Mean response curve
% ********************************************
       
  lnYh = evalfis(lnX,fismat);
      
  figure; % Figure 7a
  plot(lnX,lnY,'or', lnX, lnYh,'-b');
  line([1.57542247665, 1.57542247665], [0, 3], 'Color', [0.2 0.2 0.2]);
  ylabel('log(Aboveground Biomass)');
  xlabel('u'); 
  legend('Observed','v_{TSK}(u,\pi)');
  title('Figure 7a');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(lnY, lnYh, 8);
  
  printdata('Table 5. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
   
  ei = lnY - lnYh;
  
  figure; % Figure 7b
  hold on
  plot(lnYh,ei,'ob');
  line([-3,7],[0,0],'Color',[0.8 0.8 0.8]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 7b');
  hold off
  
% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 7c
  h = qqplot(lnYh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 7c')
      
% ******************************************** 
% Back - Transformation - Mean response curve  
% ********************************************
   
  fc =  correction_factor_eq_56(lnY, lnYh, ei);
  
  figure; % Figure 13b
  hold on
  plot(X,Y,'oblack', X, exp(lnYh)*fc,'-m');
  line([4.83272, 4.83272], [0, 3], 'Color', [0.2 0.2 0.2]);
  ylabel('y');
  xlabel('x'); 
  legend('Observed', 'E_{gTSK}(y|x)');
  title('Figure 13b');
  hold off
  
  bYh = [X, exp(lnYh)*fc];
  save bYhpla.txt bYh -ascii;
   
% ******************************************** 
% Back - Transformation - Mean response curve    
% Reproducibility indices
% ********************************************

  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(exp(lnY), exp(lnYh)*fc, 8);
  
  printdata('Table 9. Back - Transformation - Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************
% Back - Transformation - Mean response curve 
% Residuals
% ********************************************
   
  ei = Y - exp(lnYh);
  
  figure; % Figure 13c
  hold on
  plot(exp(lnX),ei,'ob');
  line([0 40],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 13c');
  hold off

% ********************************************
% Back - Transformation - Mean response curve
% Residuals qq-plot
% ********************************************

  figure; % Figure 13d
  h = qqplot(exp(lnX),ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 13d')
         
% ********************************************  
% Evaluate TSK MODEL FIS in continuous data
% ********************************************
 
  clnX  = min(lnX):((max(lnX)-min(lnX))/10000):max(lnX);
  clnX  = clnX';  
  
  [lnYh,g,phi,alpha] = evaltskfis(clnX,fismat,'gaussmf');
  
% ********************************************
% Membership functions
% ********************************************
      
  figure; % Figure 6a
  hold on
  plotmf(fismat,'input',1,10000);

  line([0.43282, 0.43282], [0, 1], 'Color', [0.2 0.2 0.2]);
  line([2.2521, 2.2521], [0, 1], 'Color', [0.2 0.2 0.2]);

  line([0.43282, 1.243613], [0.7, 0.7], 'Color', [0.2 0.2 0.2]);
  line([2.2521, 2.734], [0.7, 0.7], 'Color', [0.2 0.2 0.2]);
         
  ylabel('Grade of Membership');
  xlabel('u') 
  legend('\mu_1 (u)','\mu_2 (u)');
  title('Figure 6a')
  hold off
  
% ********************************************
% Normalized firing strength
% ********************************************
      
  figure; % Figure 6b
  plot(clnX,phi(:,1),'-r', clnX,phi(:,2),'-b');
  ylabel('Normalized firing strength');
  xlabel('u') ;
  legend('\vartheta^1(u)','\vartheta^2(u)');
  line([1.57542247665, 1.57542247665], [0, 0.5], 'Color', [0.2 0.2 0.2]);
  title('Figure 6b');
  
% ********************************************
% Consequent function
% ********************************************
       
  figure; % Figure 6c
  plot(clnX,g(:,1),'-r', clnX,g(:,2),'-b');
  ylabel('Consequent functions');
  xlabel('u') 
  legend('f^1(u) = ln\beta_1 + \alpha_1 u',...
         'f^2(u) = ln\beta_2 + \alpha_2 u');
  title('Figure 6c');
  
% ********************************************
% Component product
% ********************************************
    
  figure; % Figure 6d
  plot(clnX,phi(:,1).*g(:,1),'-r', clnX,phi(:,2).*g(:,2),'-b');
  ylabel('Components product');
  xlabel('u') 
  legend('\vartheta^{ 1}(u) f^{ 1}(u)',...
         '\vartheta^{ 2}(u) f^{ 2}(u)');
  title('Figure 6d');
