% AUTHORS 
% CECILIA LEAL RAMIREZ
% HECTOR A ECHAVARRIA HERAS
% JUAN RAMON CASTRO RODRIGUEZ
% ENRIQUE VILLA DIHARCE

% This program creates a Fuzzy Inference System (FIS) of 
% Takagi-Sugeno-Kang (TSK) type for the data in Robertis.

% Cleaning variables resident in the memory and screen

  clc
  clearvars;

% Reading the data of the Robertis and Williams 2008

  load DeRobertis_and_Williams_Raw_data.txt
    
  A = [DeRobertis_and_Williams_Raw_data(:,1), DeRobertis_and_Williams_Raw_data(:,2)];
    
% Data

  lnX = A(:,1);
  lnY = A(:,2);
  
% ********************************************
% Observed data
% ********************************************
       
  figure; % Figure 1d
  plot(lnX,lnY,'or');
  ylabel('log(Weight)');
  xlabel('log(Length)'); 
  title('Figure 1d');
  
% ********************************************  
% Building the TSK - PLA fuzzy inference model
% ********************************************
      
  radii = 0.5;
  
  fismat = gentskfis(lnX, lnY, radii);
   
  if (isempty(fismat) == 1)
      return;
  end
  
  printprms('Mean Response Curve', fismat);
  
% ********************************************  
% Evaluate TSK MODEL FIS
% ********************************************
 
  [lnYh,g,phi,alpha] = evaltskfis(lnX,fismat,'gaussmf');
   
% ********************************************
% Mean response curve
% ********************************************
      
  figure; % Figure 11d
  plot(lnX,lnY,'or', lnX, lnYh,'-b');
  ylabel('v_{TSK}(u,\pi)');
  xlabel('u'); 
  title('Figure 11d');
  
% ********************************************  
% Reproducibility index
% ********************************************
  
  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(lnY, lnYh, 8);
  
  printdata('Table 7. Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));

% ********************************************
% Residuals
% ********************************************
   
  ei = lnY - lnYh;
  
  figure; % Figure 11e.
  hold on
  plot(lnYh,ei,'ob');
  line([0 max(lnYh)],[0,0],'Color',[0.2 0.2 0.2]);
  xlabel('Fitted values');
  ylabel('Residuals');
  title('Figure 11e');
  hold off
  
% ********************************************
% Residuals qq-plot
% ********************************************

  figure; % Figure 11f.
  h = qqplot(lnYh,ei);
  xlabel('Theoretical Quantiles');
  ylabel('Sample Quantiles');
  title('Figure 11f')
    
% ******************************************** 
% Back - Transformation - Mean response curve  
% ********************************************
  
  fc =  correction_factor_eq_56(lnY, lnYh, ei);
  
  figure; % Figure 15d
  plot(exp(lnX),exp(lnY),'oblack', exp(lnX), exp(lnYh)*fc,'om');
  ylabel('E_{gTSK}(y|x)');
  xlabel('x'); 
  title('Figure 15d')
    
  dat = [exp(lnX), exp(lnYh)*fc];
  save brobpla.txt dat -ascii
   
% ******************************************** 
% Back - Transformation - Mean response curve    
% Reproducibility indices
% ********************************************

  [AIC, CCC, R2, SEE, MPE, MPSE] = findices(exp(lnY), exp(lnYh)*fc, 8);
  
  printdata('Table 11. Back - Transformation - Mean response curve',...
             AIC, CCC, R2, SEE, MPE, MPSE, radii, length(fismat.input.mf));
  
% ********************************************  
% Evaluate TSK MODEL FIS in continuous data
% ********************************************
 
  clnX  = min(lnX):((max(lnX)-min(lnX))/10000):max(lnX);
  clnX  = clnX';  
  
  [lnYh,g,phi,alpha] = evaltskfis(clnX,fismat,'gaussmf');
  
% ********************************************
% Membership functions
% ********************************************
    
  figure; % Figure 10a
  hold on
  plotmf(fismat,'input',1,10000);
  line([3.7135, 3.7135], [0, 1], 'Color', [0.2 0.2 0.2]);
  line([3.7135, 4.060678095], [0.647, 0.647], 'Color', [0.2 0.2 0.2]);
  text(1.2,0.4,'\theta_1')
  ylabel('Grade of membership');
  xlabel('u') 
  legend('\mu_{ 1}(u)');
  title('Figure 10a');
  hold off
  
% ********************************************
% Normalized firing strength
% ********************************************
     
  figure; % Figure 10b
  plot(clnX,phi(:,1),'-r');
  ylabel('Normalized firing strength');
  xlabel('u') 
  legend('\vartheta^{ 1}(u)');
  title('Figure 10b')
  
% ********************************************
% Consequent function
% ********************************************
     
  figure; % Figure 10c
  plot(clnX,g(:,1),'-r');
  ylabel('Consequent function');
  legend('f^1(u) = ln\beta_1+\alpha_1u')
  xlabel('u') 
  title('Figure 10c');
  
% ********************************************
% Component product
% ********************************************
  
  figure; % Figure 10d
  plot(clnX,phi(:,1).*g(:,1),'-r');
  ylabel('Component product');
  xlabel('u') 
  legend('\vartheta^{ 1}(u) g^{ 1}(u)');
  title('Figure 10d')
